/*
 * $Id: check_multiurl.c,v 1.0 2005/11/22 12:03:50 peter Exp $
 * 
 * Author: Peter Ostlin <peter@op5.se>
 * 
 */

#include "check_multiurl.h"

#if (defined (__SVR4) && defined (__sun))
int vasprintf(char **ret, const char *format, va_list args)
{
        va_list copy;
        va_copy(copy, args);

        /* Make sure it is determinate, despite manuals indicating otherwise */
        *ret = 0;

        int count = vsnprintf(NULL, 0, format, args);
        if (count >= 0) {
                char* buffer = malloc(count + 1);
                if (buffer != NULL) {
                        count = vsnprintf(buffer, count + 1, format, copy);
                        if (count < 0)
                                free(buffer);
                        else
                                *ret = buffer;
                }
        }
        va_end(args);  // Each va_start() or va_copy() needs a va_end()

        return count;
}

int asprintf(char **strp, const char *fmt, ...)
{
        int size;
        va_list args;
        va_start(args, fmt);
        size = vasprintf(strp, fmt, args);
        va_end(args);
        return size;
}
#endif

struct sys_command_struct {
	char *cmd;
	char *output;
	pid_t pid;
	int ret, len;
	int buf_mult;
	int fd, fd_err;
};

int verbose=0;
int running = 0;
int num_urls = 0;
double elapsed_time=0;
char urllist[256];
int socket_timeout=10;
//struct sys_command_struct scmd[NUM_THREADS];
//struct sys_command_struct scmd[MAX_NUM_THREADS*2];
int warn_threshold=0;
int crit_threshold=0;
struct sys_command_struct *pscmd;

/* function prototypes here */
static int fd_start_cmd(struct sys_command_struct *cs);
static void finalize(int sig);
static void print_usage();
static void print_rev();
static void print_num_threads();
static void print_help();
void socket_timeout_alarm_handler(int sig);
int process_arguments (int, char **);


void build_scmd_struct(char *url_to_list){
	int res,i;
	struct sys_command_struct *cs;
	output chld_out, chld_err;
	char *tmp_cmd;
 

	if(strlen(url_to_list)>MAX_URL_LENGTH){
		printf("Multiurl UNKNOWN - Filelist urls seems mallformed.\n");
		exit(STATE_UNKNOWN);
	}
	asprintf(&tmp_cmd,"/usr/bin/wget -O- %s -q\n", url_to_list);
	
	res=np_runcmd(tmp_cmd, &chld_out, &chld_err, 0);
	if(res!=0){
		printf("Multiurl UNKNOWN - Failed to fetch urllist\n");
		exit(STATE_UNKNOWN);
	}

	num_urls = 0;
	if(chld_out.lines>MAX_URLS){
		printf("Multiurl UNKNOWN - Urlist exeeds size limit (%i)\n",MAX_URLS);
		exit(STATE_UNKNOWN);		
	}
	pscmd = (struct sys_command_struct*) malloc(chld_out.lines*sizeof(struct sys_command_struct));

	for(i = 0; i < chld_out.lines; i++) {   
		cs = &pscmd[num_urls];

		/* Dirty, to avoid overflow when long lines... */
		if(strlen(chld_out.line[i])>MAX_URL_LENGTH){
			printf("Multiurl UNKNOWN - Mallformed urllist\n");
			exit(STATE_UNKNOWN);
		}

		if(strlen(chld_out.line[i])>1){
			memset(cs, 0, sizeof(struct sys_command_struct));
			asprintf (&cs->cmd, "/opt/plugins/check_http -H %s",chld_out.line[i]);
			num_urls++;
		}
	}
}


void gather_output(struct sys_command_struct *cs){
	char buf[4096];
	int rd;
	struct timeval to = { 1, 0 };

	fd_set rs;
	int nfound;
	struct pollfd fds;

	if(cs->fd == -1) return;

	fds.fd = cs->fd;
	fds.events = POLLIN | POLLPRI;
	FD_ZERO(&rs);
	FD_SET(cs->fd, &rs);
	//   printf("gathering output on fd %d, pid: %d\n", cs->fd, cs->pid);

	while((nfound = poll(&fds, 1, 1000)) != 0) {
		//      printf("poll returned %d\n", nfound);
		to.tv_sec = 1;
		to.tv_usec = 0;
		rd = read(cs->fd, buf, sizeof(buf));
		if(!rd)
			return;
		//      printf("read() returned %d\n", rd);
		if(rd == -1) { /* an error occurred */
			if(errno == EINTR) {
				//	    printf("read() interrupted by signal\n");
				continue;
			}
			else {
				perror("read()");
				return;
			}
		}
		else if(rd > 1) { /* read went well */
			cs->output = realloc(cs->output, rd + cs->len + 1);
			memcpy(&cs->output[cs->len], buf, rd);
			cs->len += rd;
			cs->output[cs->len + 1] = '\0';
		}
	}
}

void sighandler(int sig){
	printf("Caught sig %d\n", sig);
	exit (STATE_UNKNOWN);	
}

void socket_timeout_alarm_handler (int sig){
	/* print the message */
	printf("Socket timeout after %d seconds\n", socket_timeout);
	exit (STATE_UNKNOWN);
}

int check_completion(struct sys_command_struct *cs){
	int result, status;

	if(!cs->pid) return 0;
	result = waitpid(cs->pid, &status, WNOHANG);
	
	if(result == -1)
		perror("waitpid()");

	if(result == cs->pid) {
		cs->ret = status;
		cs->pid = 0;
		gather_output(cs);
		cs->fd = -1;
	}
	return result;
}

int main(int argc, char** argv){
	int i, result, url, needed_threads;
	struct sys_command_struct *cs;
	struct timeval start_time, stop_time;
	long microsec;
   
	
	signal(SIGINT, finalize);
	// start = clock();
	#ifdef SOCKET_T
	socket_timeout=SOCKET_T;
	#endif
	process_arguments(argc,argv);   
	if(argc<3){
		print_usage();
		exit(STATE_UNKNOWN); 
	}
	
	FILE *istream;
	if ( (istream = fopen ( "/opt/plugins/check_http", "r" ) ) == NULL )
	{
		printf ( "UNKNOWN - Plugin /opt/plugins/check_http not found\n" );
		exit(3);
	}
	fclose ( istream );
	
	build_scmd_struct(urllist);

	needed_threads = (num_urls>MAX_NUM_THREADS) ? MAX_NUM_THREADS : num_urls;
   
	/* Setup alarm signal handler and start timer */
	(void) signal (SIGALRM, socket_timeout_alarm_handler);
	(void) alarm (socket_timeout);

	gettimeofday(&start_time, NULL);
   
	for(i = 0; i < needed_threads; i++) {
		cs = &pscmd[i];
		/* cs->cmd = COMMAND_TO_RUN; */

		result = fd_start_cmd(cs);
		if(result == -1)
			printf("fd_start_cmd returned -1: %s\n", strerror(errno));
		else running++;
	}
   
   	url=needed_threads;
   	while(running > 0) for(i = 0; i < num_urls; i++) {
		cs = &pscmd[i];
		gather_output(cs);
		result = check_completion(cs);
		if(result == -1) {
			perror("check_completion()");
		}
		/* Thread completed, start new if needed */
		else if(result > 0) {
			running--;
			if(url<num_urls){
				cs = &pscmd[url];
				result = fd_start_cmd(cs);
				if(result == -1)
					printf("fd_start_cmd returned -1: %s\n", strerror(errno));
				else { 
					running++;
					url++;
				}
			}
		}
	}
    
	gettimeofday (&stop_time, NULL);
	microsec = (stop_time.tv_sec - start_time.tv_sec)*1000000 + stop_time.tv_usec - start_time.tv_usec;
	elapsed_time= (double)microsec / 1.0e6;
	finalize(0);
	return 0;
}

static int fd_start_cmd(struct sys_command_struct *cs){
	int pfd[2], pfderr[2];
	char *env[2];
	env[0] = strdup("LC_ALL=C");
	env[1] = '\0';
	cs->output = NULL;
	if((cs->fd = np_runcmd_open(cs->cmd, pfd, pfderr, env)) == -1)
		return -1;
   
	/* fcntl(cs->fd, F_SETFL, O_NONBLOCK); */
	/* fcntl(cs->fd_err, F_SETFL, O_NONBLOCK); */
	cs->fd_err = pfderr[0];
	cs->pid = runcmd_pid(cs->fd);
	/* cs->fp = popen(cs->cmd, "r"); */
	/* if(cs->fp == NULL) return -1; */
	/* cs->fd = fileno(cs->fp); */
    
	return 0;
}

static void finalize(int sig){
	int noout, i;

	char *un_err_msg,*un_err_url;
	char *crit_err_msg, *crit_err_url;
	char *warn_err_msg, *warn_err_url;
	
	int num_ok=0,  num_crit=0, num_un=0, num_warn=0;
	if(sig) printf("Caught sig %d with finalize()\n", sig);
   	int prev_state;
	noout = 0;
	for(i = 0; i < num_urls; i++) {
		prev_state=STATE_UNKNOWN;
		
		if(pscmd[i].ret==STATE_OK || (pscmd[i].ret/256)==STATE_OK){
			num_ok++;
			prev_state=STATE_OK;
			if(verbose){
				printf("OK\t  returned by: %s\n",&pscmd[i].cmd[27]);

				}
		}
		if(pscmd[i].ret==STATE_WARNING || (pscmd[i].ret/256)==STATE_WARNING){
			num_warn++;
			prev_state=STATE_WARNING;
			asprintf(&warn_err_msg,pscmd[i].output);
			asprintf(&warn_err_url,&pscmd[i].cmd[27]);
			warn_err_msg[strlen(warn_err_msg)-1]='\0';
			warn_err_url[strlen(warn_err_url)-1]='\0';
			if(verbose){
				printf("Warning\t  returned by: %s\n",warn_err_url);
			}
		}
		if(pscmd[i].ret==STATE_CRITICAL || (pscmd[i].ret/256)==STATE_CRITICAL){
			num_crit++;
			prev_state=STATE_CRITICAL;
			asprintf(&crit_err_msg,pscmd[i].output);
			asprintf(&crit_err_url,&pscmd[i].cmd[27]);
			crit_err_msg[strlen(crit_err_msg)-1]='\0';
			crit_err_url[strlen(crit_err_url)-1]='\0';
			if(verbose)
				printf("Critical returned by: %s\n",crit_err_url);
		}

		if(prev_state==STATE_UNKNOWN){
			asprintf(&un_err_msg,pscmd[i].output);
			asprintf(&un_err_url,&pscmd[i].cmd[27]);
			un_err_msg[strlen(un_err_msg)-1]='\0';
			un_err_url[strlen(un_err_url)-1]='\0';
			if(verbose)
				printf("Unknown\t returned by: %s\n",un_err_url);
		}

/*
		puts("-------------------------------------");
      	printf("Command: %s\n", pscmd[i].cmd);
      	printf("exitcode: %d; Output (%d bytes)::\n", pscmd[i].ret, pscmd[i].len);
*/
//      if(pscmd[i].output) puts(pscmd[i].output);
//      else noout++;
   
	}
   
	num_un=num_urls - (num_ok+num_warn+num_crit); 

	if((crit_threshold != -1) && (num_crit > crit_threshold )){
		printf("Multiurl CRITICAL: %i of %i url's return CRITICAL|('%s' returns '%s\n",
			   num_crit, num_urls,crit_err_url,crit_err_msg);
		exit(STATE_CRITICAL);
	}
	if((warn_threshold != -1) && (num_warn > warn_threshold)){
		printf("Multiurl WARNING: %i of %i url's return WARNING|'%s' returns '%s\n",
			   num_warn, num_urls,warn_err_url,warn_err_msg);
		exit(STATE_WARNING);
	}
	if(num_un>0){
		printf("Multiurl UNKNOWN: %i of %i url's return UNKNOWN|'%s' returns '%s\n",
			   num_un, num_urls,un_err_url,un_err_msg);
		exit(STATE_UNKNOWN);
	}

	printf("Multiurl OK: %i of %i url's return ok| exectime: %.2f\n",num_ok,num_urls,elapsed_time);
	exit(STATE_OK);
}


/* process command-line arguments */
int process_arguments (int argc, char **argv){
	int c;
	int option = 0;
	static struct option longopts[] = {
		{"critical", required_argument, 0, 'c'},
		{"warning", required_argument, 0, 'w'},
		{"urllist", required_argument, 0, 'u'},
		{"version", no_argument, 0, 'V'},
		{"help", no_argument, 0, 'h'},
		{0, 0, 0, 0}
	};
	
	if (argc < 2) {
		print_usage ();
		exit(STATE_UNKNOWN);
	}
	
	while (1) {
		c = getopt_long (argc, argv, "+hVvtu:c:w:", longopts, &option);
		if (c == -1 || c == EOF || c == 1)
			break;
		
		switch (c) {
			case '?':   /* print short usage stateme$ */
				//usage2 (_("Unknown argument"), optarg);
				print_usage();
				exit (STATE_UNKNOWN);
			case 'h':    /* help */
				print_help ();
				exit (STATE_OK);
			case 't':    /* help */
				print_num_threads();
				exit (STATE_OK);
			case 'V':  /* version */
				print_rev ();
				exit (STATE_OK);
			    break;
			case 'u':
				strncpy(urllist,optarg,255);
				break;
			case 'w':
				warn_threshold = atoi(optarg);
				break;
			case 'c':
				crit_threshold = atoi(optarg);
				break;
			case 'v':
				verbose=1;
				break;
				
		}
	}
//	printf("w:%i, c: %i\n", warn_threshold, crit_threshold);
	return OK;
}
		

void print_help(void){
	printf("Fetches a list of url's and checks them all in paralell.\n");

	printf("Utilizes the check_http plugin.\n\n");
	print_usage();
}


void print_usage (void){
	printf("Usage: %s -u <path-to-urllist> -w <warn> -c <crit>\n\n", progname);
	printf("Example:\n");
	printf("\tcheck_multiurl -u www.myserver.se/urllist.txt -w 5 -c 8\n");
	printf("  Returns WARNING if 5 or more url's return warning\n");
	printf("  Returns CRITICAL if 8 or more url's return critical\n\n");
	printf("Urllist format: <url> [-u path] [-s search-string]\n");
	
	printf("Example: \n");
	printf("\twww.op5.se -u /Pages/startsida.htm -s OP5\n");
}
void print_rev (void){
   printf ("%s v%s\n",progname, revision);
}

void print_num_threads(){
	print_rev();
	printf("Number of concurrent threads: %i\n", MAX_NUM_THREADS);
}
