/******************************************************************************
 *
 * Nagios plugins common include file
 *
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (nagios@nagios.org)
 *
 * Last Modified: 11-05-1999
 *
 * Description:
 *
 * This file contains common include files and defines used in many of
 * the plugins.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: common.h,v 1.4 2005/06/17 13:25:24 exon Exp $
 *
 *****************************************************************************/

#ifndef NAGIOSPLUG_COMMON_H
#define NAGIOSPLUG_COMMON_H

/* For non-GNU compilers to ignore __attribute__
 * This must be here so that imported GNU headers can be used with non-gnu
 * compilers and so we can safely (and properly) put prototypes here. */
#ifndef __GNUC__
# define __attribute__(x) /* do nothing */
#endif

# include "config.h"

#ifdef HAVE_FEATURES_H
#include <features.h>
#endif

#include <stdio.h>							/* obligatory includes */
#include <stdlib.h>
#include <errno.h>

#ifdef HUGE_VAL_NEEDS_MATH_H
#include <math.h>
#endif

#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# ifdef HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif

#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif

#include <getopt.h>
#include <ctype.h>

#ifdef HAVE_LWRES_NETDB_H
#include <lwres/netdb.h>
#else
# if !HAVE_GETADDRINFO
#  include "getaddrinfo.h"
# else
#  include <netdb.h>
# endif
#endif

#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif

#ifdef HAVE_DECL_SWAPCTL
# ifdef HAVE_SYS_SWAP_H
#  include <sys/swap.h>
# endif
# ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
# endif
# ifdef HAVE_SYS_PARAM_H
#  include <sys/param.h>
# endif
#endif

#ifndef SWAP_CONVERSION
# define SWAP_CONVERSION 1
#endif

/*
 *
 * Missing Functions
 *
 */

#ifndef HAVE_STRTOL
# define strtol(a,b,c) atol((a))
#endif

#ifndef HAVE_STRTOUL
# define strtoul(a,b,c) (unsigned long)atol((a))
#endif

#ifndef HAVE_ASPRINTF
int asprintf(char **strp, const char *fmt, ...)
	__attribute__((__format__(__printf__, 2, 3)));
#endif

/*
#ifndef HAVE_VASPRINTF
int vasprintf(char **strp, const char *fmt, va_list ap)
	__attribute__((__format__(__printf__, 2, 0)));
#endif
*/

#ifndef HAVE_SNPRINTF
int snprintf(char *str, size_t size, const  char  *format, ...)
	__attribute__((__format__(__printf__, 3, 4)));
#endif

#ifndef HAVE_VSNPRINTF
int vsnprintf(char *str, size_t size, const char  *format, va_list ap)
	__attribute__((__format__(__printf__, 3, 0)));
#endif

/*
 *
 * Standard Values
 *
 */

/* These should be macros so they never have to be fetched
 * from a different data-segment than the code evaluating it (which is
 * bizarrely expensive on some weird architectures). It also makes it
 * easier to develop and implement additional API's without mucking about
 * with the "official" code (#ifdef STATE_CRITICAL etc).
 */
#define OK 0
#define ERROR -1

/* AIX seems to have this defined somewhere else */
#ifndef FALSE
# define FALSE 0
# define TRUE 1
#endif

enum {
	STATE_OK,
	STATE_WARNING,
	STATE_CRITICAL,
	STATE_UNKNOWN,
	STATE_DEPENDENT,
	STATE_OOB		/* status out of bounds (keep last in list!) */
};

#define DEFAULT_SOCKET_TIMEOUT 10    /* timeout after 10 seconds */
#define MAX_INPUT_BUFFER 1024        /* max size of most buffers we use */
#define MAX_HOST_ADDRESS_LENGTH 256	 /* max size of a host address */

/*
 *
 * Internationalization
 *
 */
#include "gettext.h"
#define _(String) gettext (String)

#endif /* NAGIOSPLUG_COMMON_H */
